'use strict';

var tslib = require('tslib');
var gLite = require('@antv/g-lite');
var DeviceRenderer = require('@antv/g-plugin-device-renderer');
var DomInteraction = require('@antv/g-plugin-dom-interaction');
var HTMLRenderer = require('@antv/g-plugin-html-renderer');
var ImageLoader = require('@antv/g-plugin-image-loader');
var util = require('@antv/util');
var gDeviceApi = require('@antv/g-device-api');

function _interopNamespaceDefault(e) {
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n.default = e;
    return Object.freeze(n);
}

var DeviceRenderer__namespace = /*#__PURE__*/_interopNamespaceDefault(DeviceRenderer);
var DomInteraction__namespace = /*#__PURE__*/_interopNamespaceDefault(DomInteraction);
var HTMLRenderer__namespace = /*#__PURE__*/_interopNamespaceDefault(HTMLRenderer);
var ImageLoader__namespace = /*#__PURE__*/_interopNamespaceDefault(ImageLoader);

var WebGLContextService = /** @class */ (function () {
    function WebGLContextService(context) {
        this.canvasConfig = context.config;
        // @ts-ignore
        this.deviceRendererPlugin = context.deviceRendererPlugin;
    }
    WebGLContextService.prototype.init = function () {
        var _a = this.canvasConfig, container = _a.container, canvas = _a.canvas;
        if (canvas) {
            this.$canvas = canvas;
            if (container &&
                canvas.parentElement !== container) {
                container.appendChild(canvas);
            }
            this.$container = canvas.parentElement;
            this.canvasConfig.container = this.$container;
        }
        else if (container) {
            // create container
            this.$container = util.isString(container)
                ? document.getElementById(container)
                : container;
            if (this.$container) {
                // create canvas
                var $canvas = document.createElement('canvas');
                this.$container.appendChild($canvas);
                if (!this.$container.style.position) {
                    this.$container.style.position = 'relative';
                }
                this.$canvas = $canvas;
            }
        }
        this.resize(this.canvasConfig.width, this.canvasConfig.height);
    };
    WebGLContextService.prototype.getDomElement = function () {
        return this.$canvas;
    };
    WebGLContextService.prototype.getContext = function () {
        // @ts-ignore
        return this.deviceRendererPlugin.getDevice().gl;
    };
    WebGLContextService.prototype.getBoundingClientRect = function () {
        if (this.$canvas.getBoundingClientRect) {
            return this.$canvas.getBoundingClientRect();
        }
    };
    WebGLContextService.prototype.destroy = function () {
        // @ts-ignore
        if (this.$container && this.$canvas && this.$canvas.parentNode) {
            // @ts-ignore
            this.$container.removeChild(this.$canvas);
        }
    };
    WebGLContextService.prototype.resize = function (width, height) {
        // use user-defined dpr first
        var devicePixelRatio = this.canvasConfig.devicePixelRatio;
        var dpr = devicePixelRatio || (gLite.isBrowser && window.devicePixelRatio) || 1;
        dpr = dpr >= 1 ? Math.ceil(dpr) : 1;
        this.dpr = dpr;
        if (this.$canvas) {
            var dpr_1 = this.getDPR();
            // set canvas width & height
            this.$canvas.width = dpr_1 * width;
            this.$canvas.height = dpr_1 * height;
            // set CSS style width & height
            gLite.setDOMSize(this.$canvas, width, height);
        }
    };
    WebGLContextService.prototype.getDPR = function () {
        return this.dpr;
    };
    WebGLContextService.prototype.applyCursorStyle = function (cursor) {
        if (this.$container && this.$container.style) {
            this.$container.style.cursor = cursor;
        }
    };
    WebGLContextService.prototype.toDataURL = function (options) {
        if (options === void 0) { options = {}; }
        return tslib.__awaiter(this, void 0, void 0, function () {
            return tslib.__generator(this, function (_a) {
                // @see https://stackoverflow.com/questions/26783586/canvas-todataurl-returns-blank-image
                return [2 /*return*/, this.deviceRendererPlugin.toDataURL(options)];
            });
        });
    };
    return WebGLContextService;
}());

var ContextRegisterPlugin = /** @class */ (function (_super) {
    tslib.__extends(ContextRegisterPlugin, _super);
    function ContextRegisterPlugin(rendererPlugin, config) {
        var _this = _super.call(this) || this;
        _this.rendererPlugin = rendererPlugin;
        _this.config = config;
        _this.name = 'webgl-context-register';
        return _this;
    }
    ContextRegisterPlugin.prototype.init = function () {
        this.context.ContextService = WebGLContextService;
        this.context.deviceRendererPlugin = this.rendererPlugin;
        var config = this.config;
        this.context.deviceContribution = new gDeviceApi.WebGLDeviceContribution(tslib.__assign(tslib.__assign({}, ((config === null || config === void 0 ? void 0 : config.targets)
            ? {
                targets: config.targets,
            }
            : {
                targets: ['webgl2', 'webgl1'],
            })), { onContextLost: config === null || config === void 0 ? void 0 : config.onContextLost, onContextRestored: config === null || config === void 0 ? void 0 : config.onContextRestored, onContextCreationError: config === null || config === void 0 ? void 0 : config.onContextCreationError }));
    };
    ContextRegisterPlugin.prototype.destroy = function () {
        delete this.context.ContextService;
    };
    return ContextRegisterPlugin;
}(gLite.AbstractRendererPlugin));

var WebXRManager = /** @class */ (function () {
    function WebXRManager(plugin) {
        var _this = this;
        this.plugin = plugin;
        this.onSessionEnd = function () {
            _this.session.removeEventListener('end', _this.onSessionEnd);
        };
    }
    WebXRManager.prototype.setSession = function (canvas, session) {
        return tslib.__awaiter(this, void 0, void 0, function () {
            var gl, attributes, layerInit, _a;
            return tslib.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        if (!session) return [3 /*break*/, 5];
                        this.session = session;
                        gl = this.plugin.getDevice()['gl'];
                        attributes = gl.getContextAttributes();
                        if (!(attributes.xrCompatible !== true)) return [3 /*break*/, 2];
                        // @see https://developer.mozilla.org/en-US/docs/Web/API/WebGLRenderingContext/makeXRCompatible
                        return [4 /*yield*/, gl.makeXRCompatible()];
                    case 1:
                        // @see https://developer.mozilla.org/en-US/docs/Web/API/WebGLRenderingContext/makeXRCompatible
                        _b.sent();
                        _b.label = 2;
                    case 2:
                        // session.addEventListener('select', this.onSessionEvent);
                        session.addEventListener('end', this.onSessionEnd);
                        if (!(session.renderState.layers === undefined)) return [3 /*break*/, 4];
                        layerInit = {
                            antialias: attributes.antialias,
                            alpha: true,
                            depth: attributes.depth,
                            stencil: attributes.stencil,
                            framebufferScaleFactor: 1.0,
                        };
                        this.glBaseLayer = new XRWebGLLayer(session, gl, layerInit);
                        session.updateRenderState({ baseLayer: this.glBaseLayer });
                        _a = this;
                        return [4 /*yield*/, session.requestReferenceSpace(this.referenceSpaceType)];
                    case 3:
                        _a.referenceSpace = _b.sent();
                        // @ts-ignore
                        session.referenceSpace = this.referenceSpace;
                        _b.label = 4;
                    case 4:
                        canvas.requestAnimationFrame =
                            session.requestAnimationFrame.bind(session);
                        _b.label = 5;
                    case 5: return [2 /*return*/];
                }
            });
        });
    };
    WebXRManager.prototype.setReferenceSpaceType = function (referenceSpaceType) {
        this.referenceSpaceType = referenceSpaceType;
    };
    return WebXRManager;
}());

/**
 * @see https://github.com/mrdoob/three.js/blob/master/examples/jsm/webxr/ARButton.js
 * @example
 *
 * import { ARButton } from '@antv/g-webgl';
 * const $button = ARButton.createButton(renderer, { domOverlay: { root: document.body } });
 */
var ARButton = /** @class */ (function () {
    function ARButton() {
    }
    ARButton.createButton = function (canvas, renderer, sessionInit) {
        var _this = this;
        if (sessionInit === void 0) { sessionInit = {}; }
        var button = document.createElement('button');
        var disableButton = function () {
            button.style.display = '';
            button.style.cursor = 'auto';
            button.style.left = 'calc(50% - 75px)';
            button.style.width = '150px';
            button.onmouseenter = null;
            button.onmouseleave = null;
            button.onclick = null;
        };
        var showARNotSupported = function () {
            disableButton();
            button.textContent = 'AR NOT SUPPORTED';
        };
        var showARNotAllowed = function (exception) {
            disableButton();
            console.warn('Exception when trying to call xr.isSessionSupported', exception);
            button.textContent = 'AR NOT ALLOWED';
        };
        var stylizeElement = function (element) {
            element.style.position = 'absolute';
            element.style.bottom = '20px';
            element.style.padding = '12px 6px';
            element.style.border = '1px solid #fff';
            element.style.borderRadius = '4px';
            element.style.background = 'rgba(0,0,0,0.1)';
            element.style.color = '#fff';
            element.style.font = 'normal 13px sans-serif';
            element.style.textAlign = 'center';
            element.style.opacity = '0.5';
            element.style.outline = 'none';
            element.style.zIndex = '999';
        };
        var showStartAR = function () {
            var currentSession;
            button.style.display = '';
            button.style.cursor = 'pointer';
            button.style.left = 'calc(50% - 50px)';
            button.style.width = '100px';
            button.textContent = 'START AR';
            var onSessionEnded = function () {
                currentSession.removeEventListener('end', onSessionEnded);
                button.textContent = 'START AR';
                sessionInit.domOverlay.root.style.display = 'none';
                currentSession = undefined;
            };
            var onSessionStarted = function (session) { return tslib.__awaiter(_this, void 0, void 0, function () {
                return tslib.__generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            session.addEventListener('end', onSessionEnded);
                            renderer.xr.setReferenceSpaceType('local');
                            return [4 /*yield*/, renderer.xr.setSession(canvas, session)];
                        case 1:
                            _a.sent();
                            button.textContent = 'STOP AR';
                            sessionInit.domOverlay.root.style.display = '';
                            currentSession = session;
                            return [2 /*return*/];
                    }
                });
            }); };
            if (sessionInit.domOverlay === undefined) {
                var overlay = document.createElement('div');
                overlay.style.display = 'none';
                document.body.appendChild(overlay);
                var svg = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
                svg.setAttribute('width', '38');
                svg.setAttribute('height', '38');
                svg.style.position = 'absolute';
                svg.style.right = '20px';
                svg.style.top = '20px';
                svg.addEventListener('click', function () {
                    currentSession.end();
                });
                overlay.appendChild(svg);
                var path = document.createElementNS('http://www.w3.org/2000/svg', 'path');
                path.setAttribute('d', 'M 12,12 L 28,28 M 28,12 12,28');
                path.setAttribute('stroke', '#fff');
                path.setAttribute('stroke-width', '2');
                svg.appendChild(path);
                if (sessionInit.optionalFeatures === undefined) {
                    sessionInit.optionalFeatures = [];
                }
                sessionInit.optionalFeatures.push('dom-overlay');
                sessionInit.domOverlay = { root: overlay };
            }
            /**
             * Bind event listeners to button.
             */
            button.onclick = function () {
                if (!currentSession) {
                    navigator.xr
                        .requestSession('immersive-ar', sessionInit)
                        .then(onSessionStarted);
                }
                else {
                    currentSession.end();
                }
            };
            button.onmouseenter = function () {
                button.style.opacity = '1.0';
            };
            button.onmouseleave = function () {
                button.style.opacity = '0.5';
            };
        };
        if ('xr' in navigator) {
            button.id = 'ARButton';
            button.style.display = 'none';
            stylizeElement(button);
            navigator.xr
                .isSessionSupported('immersive-ar')
                .then(function (supported) {
                supported ? showStartAR() : showARNotSupported();
            })
                .catch(showARNotAllowed);
            return button;
        }
        else {
            var message = document.createElement('a');
            if (window.isSecureContext === false) {
                message.href = document.location.href.replace(/^http:/, 'https:');
                message.innerHTML = 'WEBXR NEEDS HTTPS'; // TODO Improve message
            }
            else {
                message.href = 'https://immersiveweb.dev/';
                message.innerHTML = 'WEBXR NOT AVAILABLE';
            }
            message.style.left = 'calc(50% - 90px)';
            message.style.width = '180px';
            message.style.textDecoration = 'none';
            stylizeElement(message);
            return message;
        }
    };
    return ARButton;
}());

var Renderer = /** @class */ (function (_super) {
    tslib.__extends(Renderer, _super);
    function Renderer(config) {
        var _this = _super.call(this, tslib.__assign({ enableSizeAttenuation: false }, config)) || this;
        var deviceRendererPlugin = new DeviceRenderer__namespace.Plugin(config);
        _this.xr = new WebXRManager(deviceRendererPlugin);
        _this.registerPlugin(new ContextRegisterPlugin(deviceRendererPlugin, config));
        _this.registerPlugin(new ImageLoader__namespace.Plugin());
        _this.registerPlugin(deviceRendererPlugin);
        _this.registerPlugin(new DomInteraction__namespace.Plugin());
        _this.registerPlugin(new HTMLRenderer__namespace.Plugin());
        return _this;
    }
    return Renderer;
}(gLite.AbstractRenderer));

exports.DeviceRenderer = DeviceRenderer__namespace;
exports.DomInteraction = DomInteraction__namespace;
exports.HTMLRenderer = HTMLRenderer__namespace;
exports.ARButton = ARButton;
exports.Renderer = Renderer;
exports.WebXRManager = WebXRManager;
//# sourceMappingURL=index.js.map
